from pathlib import Path
from subprocess import Popen, PIPE
import uvicorn
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from pydantic import BaseModel
import uuid

app = FastAPI()


class CMD(BaseModel):
    cmd: str


class PYCMD(BaseModel):
    exec_path: str
    py_scripts: str


origins = [
    "http://localhost:8080",
    "https://localhost:3000"
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


@app.post("/sh")
def exec_sh(cmd: CMD) -> bytes:
    popen = Popen(cmd.cmd, stdout=PIPE, stderr=PIPE, shell=True)
    popen.wait()
    error = popen.stderr.read()
    stdout = popen.stdout.read()
    if error:
        raise HTTPException(status_code=500, detail=error)
    return stdout


@app.post("/py")
def exec_py(py_cmd: PYCMD) -> bytes:
    fid = uuid.uuid4()
    fpath = Path(f"/tmp/{fid}")
    fpath.write_text(py_cmd.py_scripts)
    exec_path = Path(py_cmd.exec_path).absolute()
    popen = Popen(f"python {fpath}", stdout=PIPE, stderr=PIPE, shell=True, cwd=exec_path)
    popen.wait()
    error = popen.stderr.read()
    stdout = popen.stdout.read()
    if error:
        raise HTTPException(status_code=500, detail=error)
    return stdout


if __name__ == '__main__':
    uvicorn.run("remote_sh:app", port=8892, host="0.0.0.0")


from pathlib import Path
from typing import Union

from pymongo import MongoClient


def create(_id, program, script, settings: Union[dict, str], dbName, collectionName, host, port):
    client = MongoClient(host=host, port=port)
    db = client[dbName]
    collection = db[collectionName]
    return collection.insert_one({
        "_id": _id,
        "program": program,
        "script": script,
        "settings": settings
    })


if __name__ == '__main__':
    res = create("6",
                 "python",
                 Path("stepScript.py").read_text(),
                 settings={
                     "a": 10
                 },
                 dbName="test",
                 collectionName="steps",
                 host="42.244.24.75",
                 port=27000
                 )
    print(res)



import textwrap
from pathlib import Path
from subprocess import Popen


def writeIncar(directory, **kwargs):
    """VASP输入文件
    :kwargs 字典形式的vasp参数，忽略大小写
    """
    path = Path(directory)
    INCAR = path / "INCAR" if path.name != "INCAR" else path
    INCAR.parent.mkdir(parents=True, exist_ok=True)
    with open(INCAR, "w+") as fd:
        for k, v in kwargs.items():
            if isinstance(v, bool):
                if v is True:
                    kwargs[k] = ".TRUE."
                elif v is False:
                    kwargs[k] = ".FALSE."
            if isinstance(v, (list, tuple)):
                v = [str(i) for i in v]
                kwargs[k] = " ".join(v)
            if type(v) is int or type(v) is float:
                kwargs[k] = str(v)
            fd.write(f"{k.upper().ljust(8)} = {v}\n")


def writeKpoints(directory, mode="Monkhorst-Pack", subdivisions=None, kpaths=None, specialKpoints=None, density=0,
                 vaspkit=None):
    """VASP KPOINTS Writer
    :param specialKpoints: Special Kpoints
    :param vaspkit: generate KPOINTS by vaspkit
    :param directory: Write Path
    :param mode: Monkhorst-Pack | BandStructure
    :param subdivisions: subdivisions N_1, N_2 and N_3 along recipr. latt. vectors
    :param kpaths: BandStructure KPath
    :param density: Number of k-points
    Example:
    For structure optimizition:
        writeKpoints(
            directory=".",
            mode="Monkhorst-Pack",
            subdivisions=(6,6,6),
            density=10
        )
        You will get a KPOINTS file like this:
            KPOINTS Generated by HTSCT
            10
            Gamma
            6 1 1
            0 0 0
    For bandstructure:
        writeKpoints(
            directory=".",
            mode="BandStructure",
            kpaths=["GAMMA", "X", "GAMMA"],
            specialKpoints={"GAMMA": (0.0, 0.0, 0.0), "X": (0.5, 0.0, 0.0)},
            density=20
        )
        You will get a KPOINTS file like this:
            K-PATH Generated by HTSCT
               20
            Line-Mode
            Reciprocal
               0.0000000000   0.0000000000   0.0000000000      GAMMA
               0.5000000000   0.0000000000   0.0000000000      X

               0.5000000000   0.0000000000   0.0000000000      X
               0.0000000000   0.0000000000   0.0000000000      GAMMA
    """
    path = Path(directory)
    KPOINTS = path / "KPOINTS" if path.name != "KPOINTS" else path
    KPOINTS.parent.mkdir(parents=True, exist_ok=True)
    if vaspkit:
        if vaspkit == "1D":
            Popen("vaspkit -task 301\n", cwd=directory).wait()
        elif vaspkit == "2D":
            Popen("vaspkit -task 302\n", cwd=directory).wait()
        elif vaspkit == "3D":
            Popen("vaspkit -task 303\n", cwd=directory).wait()
        else:
            raise Exception("vaspkit can only be set to 1D|2D|3D")
        (directory / "KPATH.in").replace(directory / "KPOINTS")
        return

    if mode == "Monkhorst-Pack":
        subdivisions = map(lambda x: str(x), subdivisions)
        content = textwrap.dedent(f"""
        KPOINTS Generated by HTSCT
        {density}
        Gamma
        {" ".join(subdivisions)}
        0 0 0
        """).lstrip()
        with open(KPOINTS, "w+") as fd:
            fd.write(content)
    if mode == "BandStructure":
        _kpaths = ""
        for i in range(len(kpaths)):
            if i < len(kpaths) - 1:
                k1 = kpaths[i]
                k2 = kpaths[i + 1]
                v1 = map(lambda x: str(x).ljust(12, "0").rjust(15), specialKpoints[k1])
                v2 = map(lambda x: str(x).ljust(12, "0").rjust(15), specialKpoints[k2])
                _kpaths += f"{''.join(v1)}{' ' * 6}{k1}\n{''.join(v2)}{' ' * 6}{k2}\n\n"
        content = textwrap.dedent(f"""
        K-PATH Generated by HTSCT
           {density}
        Line-Mode
        Reciprocal
        """).lstrip() + _kpaths
        with open(KPOINTS, "w+") as fd:
            fd.write(content)


def writeOptcell(directory, a=None, b=None, c=None):
    """
    Help to write the vasp OPTCELL file.
    :param directory: Write Path
    :param a:a axis
    :param b:b axis
    :param c:c axis
    """
    default = ("0", "0", "0")
    path = Path(directory)
    OPTCELL = path / "OPTCELL" if path.name != "OPTCELL" else path
    OPTCELL.parent.mkdir(parents=True, exist_ok=True)
    _a = list(map(lambda x: str(x), a)) if a else default
    _b = list(map(lambda x: str(x), b)) if b else default
    _c = list(map(lambda x: str(x), c)) if c else default
    content = f"{' '.join(_a)}\n{' '.join(_b)}\n{' '.join(_c)}"
    with open(OPTCELL, "w+") as fd:
        fd.write(content)


def writePotcar(directory):
    Popen("vaspkit -task 103\n", cwd=directory, shell=True).wait()  # 生成POTCAR


if __name__ == '__main__':
    writeKpoints(
        directory=".",
        subdivisions=(1, 1, 1)
    )



from argparse import ArgumentParser
from subprocess import Popen, PIPE

script = """
        #!/bin/sh
#lipai@mail.ustc.edu.cn

begin=$1
if [ $begin =='' ]; then begin=0; fi    
awk -v begin=$begin '/E0/{if ( i<begin  ) i++;else print $0 }' OSZICAR >temp.e

awk '/POSITION/,/drift/{
    if(NF==6) print $4,$5,$6;
    else if($1=="total") print $1 }' OUTCAR >temp.f

awk '{if($4=="F"||$4=="T") print $4,$5,$6}' CONTCAR >temp.fix
flag=`wc temp.fix|awk '{print $1}'`
steps=`grep E0 OSZICAR |tail -1 |awk '{print $1}'`
if [ flag != '0' ] ; then
    if [ -f temp.fixx ] ; then rm temp.fixx ; fi
    for i in `seq $steps`;do
        cat temp.fix >>temp.fixx
        echo >>temp.fixx
    done
    paste temp.f temp.fixx >temp.ff
fi

awk  '{ if($1=="total") {print ++i,a;a=0}
        else {
            if($4=="F") x=0; else x=$1;
            if($5=="F") y=0; else y=$2;
            if($6=="F") z=0; else z=$3;
            force=sqrt(x^2+y^2+z^2);
            if(a<force) a=force} }' temp.ff >force.conv

gnuplot <<EOF 
set term dumb
set title 'Energy of each ionic step'
set xlabel 'Ionic steps'
set ylabel 'Energy(eV)'
plot 'temp.e' u 1:5 w l  t "Energy in eV"
set title 'Max Force of each ionic step'
set xlabel 'Ionic steps'
set ylabel 'Force (eV/Angstrom)'
plot 'force.conv' w l t "Force in eV/Angstrom"
EOF

if [ $steps -gt 8 ] ; then
tail -5 force.conv >temp.fff
tail -5 temp.e >temp.ee
gnuplot <<EOF 
set term dumb
set title 'Energy of each ionic step for the last few steps'
set xlabel 'Ionic steps'
set ylabel 'Energy(eV)'
plot 'temp.ee' u 1:5 w l  t "Energy in eV"
set title 'Max Force of each ionic step for the last few steps'
set xlabel 'Ionic steps'
set ylabel 'Force (eV/Angstrom)'
plot 'temp.fff' w l  t "Force in eV/Angstrom"
EOF
rm temp.fff temp.ee
fi

rm temp.e temp.f temp.ff temp.fix temp.fixx
        """


class CLICommand:
    """结构优化计算中查看能量和力收敛情况
    """

    @staticmethod
    def add_argments(parser: ArgumentParser):
        pass

    @staticmethod
    def run(args, parser):
        popen = Popen(script, shell=True, stdout=PIPE, stderr=PIPE)
        stdout, stderr = popen.communicate()
        stdout = stdout.decode("utf-8")
        stderr = stderr.decode("utf-8")
        print(stdout)
        print(stderr)
